@@ -602,7 +602,8 @@ int main(int argc, char *argv[])
 
 	// If stdin is a pipe, then flush the queue
 	if (is_pipe(0)) {
-		while (q_queue_length(queue) && !suspend && transport_ok)
+		while (q_queue_length(queue) && !suspend && !stop &&
+					transport_ok)
 			send_one(queue);
 	}
 
@@ -1253,12 +1254,14 @@ static int recv_msg_gss (unsigned char *header, char *msg, uint32_t *mlen)
 
 	if (utok.length < AUDIT_RMW_HEADER_SIZE) {
 		sync_error_handler ("message too short");
+		free (utok.value);
 		return -1;
 	}
 	memcpy (header, utok.value, AUDIT_RMW_HEADER_SIZE);
 
 	if (! AUDIT_RMW_IS_MAGIC (header, AUDIT_RMW_HEADER_SIZE)) {
 		sync_error_handler ("bad magic number");
+		free (utok.value);
 		return -1;
 	}
 
@@ -1266,6 +1269,7 @@ static int recv_msg_gss (unsigned char *header, char *msg, uint32_t *mlen)
 
 	if (rlen > MAX_AUDIT_MESSAGE_LENGTH) {
 		sync_error_handler ("message too long");
+		free (utok.value);
 		return -1;
 	}
 
@@ -1273,6 +1277,7 @@ static int recv_msg_gss (unsigned char *header, char *msg, uint32_t *mlen)
 
 	*mlen = rlen;
 
+	free (utok.value);
 	return 0;
 }
 #endif
