@@ -171,7 +171,7 @@ static void tty_escape(const char *s, char *dest, unsigned int len)
 	dest[j] = '\0';	/* terminate string */
 }
 
-static const char sh_set[] = "\"'`$\\";
+static const char sh_set[] = "\"'`$\\!()| ";
 static unsigned int need_shell_escape(const char *s, unsigned int len)
 {
 	unsigned int i = 0, cnt = 0;
@@ -203,8 +203,8 @@ static void shell_escape(const char *s, char *dest, unsigned int len)
 	}
 	dest[j] = '\0';	/* terminate string */
 }
-
-static const char quote_set[] = ";'\"`#$&*?[]<>{}\\";
+                                
+static const char quote_set[] = "\"'`$\\!()| ;#&*?[]<>{}";
 static unsigned int need_shell_quote_escape(const unsigned char *s, unsigned int len)
 {
 	unsigned int i = 0, cnt = 0;
@@ -318,7 +318,7 @@ static int is_hex_string(const char *str)
 /* returns a freshly malloc'ed and converted buffer */
 char *au_unescape(char *buf)
 {
-        int len, i;
+        int olen, len, i;
         char saved, *str, *ptr = buf;
 
         /* Find the end of the name */
@@ -332,9 +332,17 @@ char *au_unescape(char *buf)
                 while (isxdigit(*ptr))
                         ptr++;
         }
+	// Make the buffer based on size of original buffer.
+	// This is in case we have unexpected non-hex digit
+	// that causes truncation of the conversion and passes
+	// back a buffer that is not sized on the expectation of
+	// strlen(buf) / 2.
+	olen = strlen(buf);
+	str = malloc(olen+1);
+
         saved = *ptr;
         *ptr = 0;
-        str = strdup(buf);
+	strcpy(str, buf);
         *ptr = saved;
 
 	/* See if its '(null)' from the kernel */
@@ -355,6 +363,11 @@ char *au_unescape(char *buf)
                 ptr++;
         }
         *ptr = 0;
+	len = ptr - str - 1;
+	olen /= 2;
+	// Because *ptr is 0, writing another 0 to it doesn't hurt anything
+	if (olen > len)
+		memset(ptr, 0, olen - len);
         return str;
 }
 
@@ -644,7 +657,7 @@ static const char *print_arch(const char *val, unsigned int machine)
 		machine = audit_elf_to_machine(ival);
 	}
         if ((int)machine < 0) {
-		if (asprintf(&out, "unknown elf type(%s)", val) < 0)
+		if (asprintf(&out, "unknown-elf-type(%s)", val) < 0)
 			out = NULL;
                 return out;
         }
@@ -652,7 +665,7 @@ static const char *print_arch(const char *val, unsigned int machine)
 	if (ptr)
 	        return strdup(ptr);
 	else {
-		if (asprintf(&out, "unknown machine type(%d)", machine) < 0)
+		if (asprintf(&out, "unknown-machine-type(%d)", machine) < 0)
 			out = NULL;
                 return out;
 	}
@@ -676,7 +689,7 @@ static const char *print_ipccall(const char *val, unsigned int base)
 	if (func)
 		return strdup(func);
 	else {
-		if (asprintf(&out, "unknown ipccall(%s)", val) < 0)
+		if (asprintf(&out, "unknown-ipccall(%s)", val) < 0)
 			out = NULL;
                 return out;
 	}
@@ -700,7 +713,7 @@ static const char *print_socketcall(const char *val, unsigned int base)
 	if (func)
 		return strdup(func);
 	else {
-		if (asprintf(&out, "unknown socketcall(%s)", val) < 0)
+		if (asprintf(&out, "unknown-socketcall(%s)", val) < 0)
 			out = NULL;
                 return out;
 	}
@@ -734,7 +747,7 @@ static const char *print_syscall(const idata *id)
 		} else
                         return strdup(sys);
         } else {
-		if (asprintf(&out, "unknown syscall(%d)", syscall) < 0)
+		if (asprintf(&out, "unknown-syscall(%d)", syscall) < 0)
 			out = NULL;
 	}
 
@@ -765,7 +778,7 @@ static const char *print_exit(const char *val)
 
 static char *print_escaped(const char *val)
 {
-	const char *out;
+	char *out;
 
         if (*val == '"') {
                 char *term;
@@ -796,6 +809,42 @@ static char *print_escaped(const char *val)
 	return strdup(val); // Something is wrong with string, just send as is
 }
 
+static const char *print_escaped_ext(int type, const idata *id)
+{
+	if (id->cwd) {
+		char *str1 = NULL, *str2, *str3 = NULL, *out = NULL;
+		str2 = print_escaped(id->val);
+		if (!str2)
+			goto err_out;
+		if (*str2 != '/') {
+			str1 = print_escaped(id->cwd);
+			if (!str1)
+				goto err_out;
+			asprintf(&str3, "%s/%s", str1, str2);
+			if (!str3)
+				goto err_out;
+		} else {
+			// Check in case /home/../etc/passwd
+			if (strstr(str2, "..") == NULL)
+				return str2;
+
+			str3 = str2;
+			str2 = NULL;
+			str1 = NULL;
+		}
+		errno = 0;
+		out = realpath(str3, NULL);
+		if (errno) // If there's an error, just return the original
+			return str3;
+err_out:
+		free(str1);
+		free(str2);
+		free(str3);
+		return out;
+	} else
+		return print_escaped(id->val);
+}
+
 static const char *print_proctitle(const char *val)
 {
 	char *out = (char *)print_escaped(val);
@@ -803,6 +852,9 @@ static const char *print_proctitle(const char *val)
 		size_t len = strlen(val) / 2;
 		const char *end = out + len;
 		char *ptr = out;
+		// Proctitle has arguments separated by NUL bytes
+		// We need to write over the NUL bytes with a space
+		// so that we can see the arguments
 		while ((ptr  = rawmemchr(ptr, '\0'))) {
 			if (ptr >= end)
 				break;
@@ -961,7 +1013,7 @@ static const char *print_socket_domain(const char *val)
 	}
         str = fam_i2s(i);
         if (str == NULL) {
-		if (asprintf(&out, "unknown family(0x%s)", val) < 0)
+		if (asprintf(&out, "unknown-family(0x%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -983,7 +1035,7 @@ static const char *print_socket_type(const char *val)
 	}
         str = sock_type_i2s(type);
         if (str == NULL) {
-		if (asprintf(&out, "unknown type(%s)", val) < 0)
+		if (asprintf(&out, "unknown-type(%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -1005,7 +1057,7 @@ static const char *print_socket_proto(const char *val)
 	}
         p = getprotobynumber(proto);
         if (p == NULL) {
-		if (asprintf(&out, "unknown proto(%s)", val) < 0)
+		if (asprintf(&out, "unknown-proto(%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -1024,7 +1076,7 @@ static const char *print_sockaddr(const char *val)
         slen = strlen(val)/2;
         host = au_unescape((char *)val);
 	if (host == NULL) {
-		if (asprintf(&out, "malformed host(%s)", val) < 0)
+		if (asprintf(&out, "malformed-host(%s)", val) < 0)
 			out = NULL;
 		return out;
 	}
@@ -1033,7 +1085,7 @@ static const char *print_sockaddr(const char *val)
 
         str = fam_i2s(saddr->sa_family);
         if (str == NULL) {
-		if (asprintf(&out, "unknown family(%d)", saddr->sa_family) < 0)
+		if (asprintf(&out, "unknown-family(%d)", saddr->sa_family) < 0)
 			out = NULL;
 		free((char *)host);
 		return out;
@@ -1143,6 +1195,9 @@ static const char *print_sockaddr(const char *val)
 					  str, n->nl_family, n->nl_pid);
                         }
                         break;
+		default:
+			rc = asprintf(&out, "{ fam=%s (unsupported) }", str);
+			break;
         }
 	if (rc < 0)
 		out = NULL;
@@ -1224,7 +1279,7 @@ static const char *print_capabilities(const char *val, int base)
 	s = cap_i2s(cap);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown capability(%s%s)",
+	if (asprintf(&out, "unknown-capability(%s%s)",
 				base == 16 ? "0x" : "", val) < 0)
 		out = NULL;
 	return out;
@@ -1384,7 +1439,7 @@ static const char *print_fcntl_cmd(const char *val)
 	s = fcntl_i2s(cmd);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown fcntl command(%d)", cmd) < 0)
+	if (asprintf(&out, "unknown-fcntl-command(%d)", cmd) < 0)
 		out = NULL;
 	return out;
 }
@@ -1406,7 +1461,7 @@ static const char *print_epoll_ctl(const char *val)
 	s = epoll_ctl_i2s(cmd);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown epoll_ctl operation (%d)", cmd) < 0)
+	if (asprintf(&out, "unknown-epoll_ctl-operation(%d)", cmd) < 0)
 		out = NULL;
 	return out;
 }
@@ -1428,7 +1483,7 @@ static const char *print_clock_id(const char *val)
 		if (s != NULL)
 			return strdup(s);
 	}
-	if (asprintf(&out, "unknown clk_id (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-clk_id(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1537,7 +1592,7 @@ static const char *print_personality(const char *val)
 		} else
 			return strdup(s);
 	}
-	if (asprintf(&out, "unknown personality (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-personality(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1559,7 +1614,7 @@ static const char *print_ptrace(const char *val)
 	s = ptrace_i2s(trace);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown ptrace (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-ptrace(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1581,7 +1636,7 @@ static const char *print_prctl_opt(const char *val)
 	s = prctl_opt_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown prctl option (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-prctl-option(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1636,7 +1691,7 @@ static const char *print_rlimit(const char *val)
 		if (s != NULL)
 			return strdup(s);
 	}
-	if (asprintf(&out, "unknown rlimit (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-rlimit(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1750,7 +1805,7 @@ static const char *print_sched(const char *val)
 			strcat(buf, "|SCHED_RESET_ON_FORK");
 		return strdup(buf);
 	}
-	if (asprintf(&out, "unknown scheduler policy (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-scheduler-policy(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1775,7 +1830,7 @@ static const char *print_sock_opt_level(const char *val)
 			const char *s = socklevel_i2s(lvl);
 			if (s != NULL)
 				return strdup(s);
-			if (asprintf(&out, "unknown sockopt level (0x%s)", val) < 0)
+			if (asprintf(&out, "unknown-sockopt-level(0x%s)", val) < 0)
 				out = NULL;
 		} else
 			return strdup(p->p_name);
@@ -1805,7 +1860,7 @@ static const char *print_sock_opt_name(const char *val, int machine)
 	s = sockoptname_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown sockopt name (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-sockopt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1827,7 +1882,7 @@ static const char *print_ip_opt_name(const char *val)
 	s = ipoptname_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown ipopt name (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-ipopt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1849,7 +1904,7 @@ static const char *print_ip6_opt_name(const char *val)
 	s = ip6optname_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown ip6opt name (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-ip6opt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1871,7 +1926,7 @@ static const char *print_tcp_opt_name(const char *val)
 	s = tcpoptname_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown tcpopt name (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-tcpopt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1893,7 +1948,7 @@ static const char *print_udp_opt_name(const char *val)
 		out = strdup("UDP_CORK");
 	else if (opt == 100)
 		out = strdup("UDP_ENCAP");
-	else if (asprintf(&out, "unknown udpopt name (0x%s)", val) < 0)
+	else if (asprintf(&out, "unknown-udpopt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1915,7 +1970,7 @@ static const char *print_pkt_opt_name(const char *val)
 	s = pktoptname_i2s(opt);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown pktopt name (0x%s)", val) < 0)
+	if (asprintf(&out, "unknown-pktopt-name(0x%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -1994,7 +2049,7 @@ static const char *print_seek(const char *val)
 	}
 	str = seek_i2s(whence);
 	if (str == NULL) {
-		if (asprintf(&out, "unknown whence(%s)", val) < 0)
+		if (asprintf(&out, "unknown-whence(%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -2378,7 +2433,7 @@ static const char *print_signals(const char *val, unsigned int base)
 		if (s != NULL)
 			return strdup(s);
 	}
-	if (asprintf(&out, "unknown signal (%s%s)",
+	if (asprintf(&out, "unknown-signal(%s%s)",
 					base == 16 ? "0x" : "", val) < 0)
 		out = NULL;
 	return out;
@@ -2401,7 +2456,7 @@ static const char *print_nfproto(const char *val)
 	s = nfproto_i2s(proto);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown netfilter protocol (%s)", val) < 0)
+	if (asprintf(&out, "unknown-netfilter-protocol(%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -2423,7 +2478,7 @@ static const char *print_icmptype(const char *val)
 	s = icmptype_i2s(icmptype);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown icmp type (%s)", val) < 0)
+	if (asprintf(&out, "unknown-icmp-type(%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -2464,7 +2519,7 @@ static const char *print_hook(const char *val)
 	}
 	str = inethook_i2s(hook);
 	if (str == NULL) {
-		if (asprintf(&out, "unknown hook(%s)", val) < 0)
+		if (asprintf(&out, "unknown-hook(%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -2486,7 +2541,7 @@ static const char *print_netaction(const char *val)
 	}
 	str = netaction_i2s(action);
 	if (str == NULL) {
-		if (asprintf(&out, "unknown action(%s)", val) < 0)
+		if (asprintf(&out, "unknown-action(%s)", val) < 0)
 			out = NULL;
 		return out;
 	} else
@@ -2529,8 +2584,13 @@ static const char *print_list(const char *val)
 	if (errno) { 
 		if (asprintf(&out, "conversion error(%s)", val) < 0)
 			out = NULL;
-	} else
-		out = strdup(audit_flag_to_name(i));
+	} else {
+		const char *o = audit_flag_to_name(i);
+		if (o != NULL)
+			out = strdup(o);
+		else if (asprintf(&out, "unknown-list(%s)", val) < 0)
+			out = NULL;
+	}
 	return out;
 }
 
@@ -2689,7 +2749,7 @@ static const char *print_seccomp_code(const char *val)
 	s = seccomp_i2s(code & SECCOMP_RET_ACTION);
 	if (s != NULL)
 		return strdup(s);
-	if (asprintf(&out, "unknown seccomp code (%s)", val) < 0)
+	if (asprintf(&out, "unknown-seccomp-code(%s)", val) < 0)
 		out = NULL;
 	return out;
 }
@@ -2719,6 +2779,7 @@ const char *interpret(const rnode *r, auparse_esc_t escape_mode)
 	id.syscall = r->syscall;
 	id.a0 = r->a0;
 	id.a1 = r->a1;
+	id.cwd = r->cwd;
 	id.name = nvlist_get_cur_name(nv);
 	id.val = nvlist_get_cur_val(nv);
 	type = auparse_interp_adjust_type(r->type, id.name, id.val);
@@ -2791,6 +2852,11 @@ char *auparse_do_interpretation(int type, const idata *id,
 			const char *val = il.cur->interp_val;
 
 			if (val) {
+				// If we don't know what it is when auditd
+				// recorded it, try it again incase the
+				// libraries have been updated to support it.
+				if (strncmp(val, "unknown-", 8 ) == 0)
+					goto unknown; 
 				if (type == AUPARSE_TYPE_UID ||
 						type == AUPARSE_TYPE_GID)
 					return print_escaped(val);
@@ -2799,6 +2865,7 @@ char *auparse_do_interpretation(int type, const idata *id,
 			}
 		}
 	}
+unknown:
 
 	switch(type) {
 		case AUPARSE_TYPE_UID:
@@ -2817,9 +2884,12 @@ char *auparse_do_interpretation(int type, const idata *id,
 			out = print_exit(id->val);
 			break;
 		case AUPARSE_TYPE_ESCAPED:
+		case AUPARSE_TYPE_ESCAPED_FILE:
+			out = print_escaped_ext(type, id);
+			break;
 		case AUPARSE_TYPE_ESCAPED_KEY:
 			out = print_escaped(id->val);
-                        break;
+			break;
 		case AUPARSE_TYPE_PERM:
 			out = print_perm(id->val);
 			break;
@@ -2917,7 +2987,7 @@ char *auparse_do_interpretation(int type, const idata *id,
 			break;
         }
 
-	if (escape_mode != AUPARSE_ESC_RAW) {
+	if (escape_mode != AUPARSE_ESC_RAW && out) {
 		char *str = NULL;
 		unsigned int len = strlen(out);
 		if (type == AUPARSE_TYPE_ESCAPED_KEY) {
