@@ -448,6 +448,10 @@ AuParser_init(AuParser *self, PyObject *args, PyObject *kwds)
             PyErr_SetString(PyExc_TypeError, "source must be open file when source_type is AUSOURCE_FILE_POINTER");
             return -1;
 	}
+#if PY_MAJOR_VERSION < 3
+	int fd = fileno(fp);
+	fp = fdopen(fd, "r");
+#endif
         if ((self->au = auparse_init(source_type, fp)) == NULL) {
             //char *filename = PYSTR_ASSTRING(PyFile_Name(source));
             char *filename = "TODO";
@@ -1691,7 +1695,7 @@ PyDoc_STRVAR(get_type_name_doc,
 get_type_name() allows access to the current record type name in the\n\
 current event.\n\
 \n\
-Returns None if the record type name is unavailable.\n\
+Raises exception (LookupError) on error.\n\
 ");
 static PyObject *
 AuParser_get_type_name(AuParser *self)
@@ -1700,6 +1704,10 @@ AuParser_get_type_name(AuParser *self)
 
     PARSER_CHECK;
     name = auparse_get_type_name(self->au);
+    if (name == NULL) {
+	PyErr_SetString(PyExc_LookupError, "Not found");
+	return NULL;
+    }
     return Py_BuildValue("s", name);
 }
 
@@ -2003,7 +2011,6 @@ AuParser_get_field_int(AuParser *self)
     return NULL;
 }
 
-// FIXME: can't tell if interpret is successful, always returns some string in somewhat arbitrary format.
 PyDoc_STRVAR(interpret_field_doc,
 "interpret_field() Return an interpretation of the current field as a string that has the chosen character escaping applied.\n\
 \n\
@@ -2024,6 +2031,26 @@ AuParser_interpret_field(AuParser *self)
     return Py_BuildValue("s", value);
 }
 
+PyDoc_STRVAR(interpret_realpath_doc,
+"interpret_realpath() Return an interpretation of the current field as a realpath string that has the chosen character escaping applied.\n\
+\n\
+If the field cannot be interpreted the field is returned unmodified.\n\
+Raises exception (RuntimeError) on error\n\
+");
+static PyObject *
+AuParser_interpret_realpath(AuParser *self)
+{
+    const char *value = NULL;
+
+    PARSER_CHECK;
+    value = auparse_interpret_realpath(self->au);
+    if (value == NULL) {
+        PyErr_SetString(PyExc_RuntimeError, "'interpretation' is NULL");
+        return NULL;
+    }
+    return Py_BuildValue("s", value);
+}
+
 static
 PyGetSetDef AuParser_getseters[] = {
     {NULL}  /* Sentinel */
@@ -2089,6 +2116,7 @@ static PyMethodDef AuParser_methods[] = {
     {"get_field_type",    (PyCFunction)AuParser_get_field_type,    METH_NOARGS,  get_field_type_doc},
     {"get_field_int",     (PyCFunction)AuParser_get_field_int,     METH_NOARGS,  get_field_int_doc},
     {"interpret_field",   (PyCFunction)AuParser_interpret_field,   METH_NOARGS,  interpret_field_doc},
+    {"interpret_realpath",   (PyCFunction)AuParser_interpret_realpath, METH_NOARGS,  interpret_realpath_doc},
     {NULL, NULL}  /* Sentinel */
 };
 
