@@ -96,6 +96,7 @@ extern "C" {
 #define AUDIT_MAC_CHECK		1134    /* User space MAC decision results */
 #define AUDIT_ACCT_LOCK		1135    /* User's account locked by admin */
 #define AUDIT_ACCT_UNLOCK	1136    /* User's account unlocked by admin */
+#define AUDIT_USER_DEVICE	1137	/* User space hotplug device changes */
 
 #define AUDIT_FIRST_DAEMON	1200
 #define AUDIT_LAST_DAEMON	1299
@@ -215,6 +216,11 @@ extern "C" {
 #define AUDIT_VIRT_CONTROL		2500 /* Start, Pause, Stop VM */
 #define AUDIT_VIRT_RESOURCE		2501 /* Resource assignment */
 #define AUDIT_VIRT_MACHINE_ID		2502 /* Binding of label to VM */
+#define AUDIT_VIRT_INTEGRITY_CHECK	2503 /* Guest integrity results */
+#define AUDIT_VIRT_CREATE		2504 /* Creation of guest image */
+#define AUDIT_VIRT_DESTROY		2505 /* Destruction of guest image */
+#define AUDIT_VIRT_MIGRATE_IN		2506 /* Inbound guest migration info */
+#define AUDIT_VIRT_MIGRATE_OUT		2507 /* Outbound guest migration info */
 
 #define AUDIT_LAST_VIRT_MSG		2599
 
@@ -256,6 +262,14 @@ extern "C" {
 #define AUDIT_FEATURE_CHANGE	1328 /* Audit feature changed value */
 #endif
 
+#ifndef AUDIT_REPLACE
+#define AUDIT_REPLACE           1329 /* Auditd replaced because probe failed */
+#endif
+
+#ifndef AUDIT_KERN_MODULE
+#define AUDIT_KERN_MODULE	1330 /* Kernel Module events */
+#endif
+
 #ifndef AUDIT_ANOM_LINK
 #define AUDIT_ANOM_LINK		1702 /* Suspicious use of file links */
 #endif
