@@ -100,6 +100,8 @@ static int space_action_parser(struct nv_pair *nv, int line,
 		struct daemon_conf *config);
 static int action_mail_acct_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
+static int verify_email_parser(struct nv_pair *nv, int line, 
+		struct daemon_conf *config);
 static int admin_space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
 static int admin_space_left_action_parser(struct nv_pair *nv, int line, 
@@ -151,6 +153,7 @@ static const struct kw_pair keywords[] =
   {"space_left",               space_left_parser,		0 },
   {"space_left_action",        space_action_parser,		1 },
   {"action_mail_acct",         action_mail_acct_parser,		0 },
+  {"verify_email",             verify_email_parser,		0 },
   {"admin_space_left",         admin_space_left_parser,		0 },
   {"admin_space_left_action",  admin_space_left_action_parser,	1 },
   {"disk_full_action",         disk_full_action_parser,		1 },
@@ -271,6 +274,7 @@ void clear_config(struct daemon_conf *config)
 	config->space_left_action = FA_IGNORE;
 	config->space_left_exe = NULL;
 	config->action_mail_acct = strdup("root");
+	config->verify_email = 1;
 	config->admin_space_left= 0L;
 	config->admin_space_left_action = FA_IGNORE;
 	config->admin_space_left_exe = NULL;
@@ -841,8 +845,7 @@ static int log_format_parser(struct nv_pair *nv, int line,
 			if (config->log_format == LF_NOLOG) {
 				audit_msg(LOG_WARNING,
 				    "The NOLOG option to log_format is deprecated. Please use the write_logs option.");
-				if (config->log_format == LF_NOLOG &&
-					config->write_logs != 0)
+				if (config->write_logs != 0)
 					audit_msg(LOG_WARNING,
 					    "The NOLOG option is overriding the write_logs current setting.");
 				config->write_logs = 0;
@@ -1111,7 +1114,7 @@ static int action_mail_acct_parser(struct nv_pair *nv, int line,
 	if (tmail == NULL)
 		return 1;
 
-	if (validate_email(tmail) > 1) {
+	if (config->verify_email && validate_email(tmail) > 1) {
 		free(tmail);
 		return 1;
 	}
@@ -1123,6 +1126,24 @@ static int action_mail_acct_parser(struct nv_pair *nv, int line,
 	return 0;
 }
 
+static int verify_email_parser(struct nv_pair *nv, int line,
+	struct daemon_conf *config)
+{
+	unsigned long i;
+	audit_msg(LOG_DEBUG, "verify_email_parser called with: %s",
+		  nv->value);
+
+
+	for (i=0; yes_no_values[i].name != NULL; i++) {
+		if (strcasecmp(nv->value, yes_no_values[i].name) == 0) {
+			config->verify_email = yes_no_values[i].option;
+			return 0;
+		}
+	}
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
+	return 1;
+}
+
 static int admin_space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
@@ -1703,12 +1724,14 @@ const char *audit_lookup_format(int fmt)
 int create_log_file(const char *val)
 {
 	int fd;
+	mode_t u;
 
-	umask(S_IRWXO);
+	u = umask(S_IRWXO);
 	fd = open(val, O_CREAT|O_EXCL|O_APPEND, S_IRUSR|S_IWUSR|S_IRGRP);
 	if (fd < 0) 
 		audit_msg(LOG_ERR, "Unable to create %s (%s)", val,
 			strerror(errno));
+	umask(u);
 	return fd;
 }
 
