@@ -1,5 +1,5 @@
 /* auditd-dispatch.c -- 
- * Copyright 2005-07,2013,2016 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-07,2013,2016-17 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,9 +36,8 @@
 
 /* This is the communications channel between auditd & the dispatcher */
 static int disp_pipe[2] = {-1, -1};
-static pid_t pid = 0;
+static volatile pid_t pid = 0;
 static int n_errs = 0;
-static int protocol_ver = AUDISP_PROTOCOL_VER;
 #define REPORT_LIMIT 10
 
 int dispatcher_pid(void)
@@ -48,7 +47,7 @@ int dispatcher_pid(void)
 
 void dispatcher_reaped(void)
 {
-	audit_msg(LOG_INFO, "dispatcher %d reaped\n", pid);
+	audit_msg(LOG_INFO, "dispatcher %d reaped", pid);
 	pid = 0;
 	shutdown_dispatcher();
 }
@@ -58,8 +57,11 @@ static int set_flags(int fn, int flags)
 {
 	int fl;
 
+	if (fn == -1)
+		return 0;
+
 	if ((fl = fcntl(fn, F_GETFL, 0)) < 0) {
-		audit_msg(LOG_ERR, "fcntl failed. Cannot get flags (%s)\n", 
+		audit_msg(LOG_ERR, "fcntl failed. Cannot get flags (%s)", 
 			strerror(errno));
 		return fl;
 	}
@@ -69,11 +71,27 @@ static int set_flags(int fn, int flags)
 	return fcntl(fn, F_SETFL, fl);
 }
 
+/*
+ * This function exists in order to prevent the dispatcher's read pipe
+ * from being leaked into other child processes. We cannot mark it
+ * CLOEXEC until after the dispatcher is started by execl or it'll
+ * get closed such that the dispatcher has no stdin fd. So, any path
+ * that leads to calling init_dispatcher needs to call this function later
+ * after we are sure the execl should have happened. Everything is serialized
+ * with the main thread, so there shouldn't be any unexpected execs.
+ */
+int make_dispatcher_fd_private(void)
+{
+	if (set_flags(disp_pipe[0], FD_CLOEXEC) < 0) {
+		audit_msg(LOG_ERR, "Failed to set FD_CLOEXEC flag");
+		return 1;
+	}
+	return 0;
+}
+
 /* This function returns 1 on error & 0 on success */
 int init_dispatcher(const struct daemon_conf *config)
 {
-	struct sigaction sa;
-
 	if (config->dispatcher == NULL) 
 		return 0;
 
@@ -82,13 +100,13 @@ int init_dispatcher(const struct daemon_conf *config)
 		return 1;
 	}
 
-	/* If the events have enriched data, we are protocol 2 */
-	if (config->log_format == LF_ENRICHED)
-		protocol_ver = AUDISP_PROTOCOL_VER2;
-	else
-		protocol_ver = AUDISP_PROTOCOL_VER;
+	/* Don't let this leak to anything */
+	if (set_flags(disp_pipe[1], FD_CLOEXEC) < 0) {
+		audit_msg(LOG_ERR, "Failed to set FD_CLOEXEC flag");
+		return 1;
+	}
 
-	/* Make both disp_pipe non-blocking */
+	/* Make both disp_pipe non-blocking if requested */
 	if (config->qos == QOS_NON_BLOCKING) {
 		if (set_flags(disp_pipe[0], O_NONBLOCK) < 0 ||
 			set_flags(disp_pipe[1], O_NONBLOCK) < 0) {
@@ -101,12 +119,8 @@ int init_dispatcher(const struct daemon_conf *config)
 	pid = fork();
 	switch(pid) {
 		case 0:	// child
-			dup2(disp_pipe[0], 0);
-			close(disp_pipe[0]);
-			close(disp_pipe[1]);
-			sigfillset (&sa.sa_mask);
-			sigprocmask (SIG_UNBLOCK, &sa.sa_mask, 0);
-			setsid();
+			if (disp_pipe[0] != 0)
+				dup2(disp_pipe[0], 0);
 			execl(config->dispatcher, config->dispatcher, NULL);
 			audit_msg(LOG_ERR, "exec() failed");
 			exit(1);
@@ -115,14 +129,6 @@ int init_dispatcher(const struct daemon_conf *config)
 			return 1;
 			break;
 		default:	// parent
-			close(disp_pipe[0]);
-			disp_pipe[0] = -1;
-			/* Avoid leaking this */
-			if (fcntl(disp_pipe[1], F_SETFD, FD_CLOEXEC) < 0) {
-				audit_msg(LOG_ERR,
-					"Failed to set FD_CLOEXEC flag");
-				return 1;
-			}
 			audit_msg(LOG_INFO, "Started dispatcher: %s pid: %u",
 					config->dispatcher, pid);
 			break;
@@ -134,11 +140,10 @@ int init_dispatcher(const struct daemon_conf *config)
 void shutdown_dispatcher(void)
 {
 	// kill child
-	if (pid)
+	if (pid) {
 		kill(pid, SIGTERM);
-	// wait for term
-	// if not in time, send sigkill
-	pid = 0;
+		pid = 0;
+	}
 
 	// cleanup comm pipe
 	if (disp_pipe[0] >= 0) {
@@ -158,15 +163,10 @@ void reconfigure_dispatcher(const struct daemon_conf *config)
 		kill(pid, SIGHUP);
 	else
 		init_dispatcher(config);
-
-	if (config->log_format == LF_ENRICHED)
-		protocol_ver = AUDISP_PROTOCOL_VER2;
-	else
-		protocol_ver = AUDISP_PROTOCOL_VER;
 }
 
 /* Returns -1 on err, 0 on success, and 1 if eagain occurred and not an err */
-int dispatch_event(const struct audit_reply *rep, int is_err)
+int dispatch_event(const struct audit_reply *rep, int is_err, int protocol_ver)
 {
 	int rc, count = 0;
 	struct iovec vec[2];
