@@ -31,9 +31,11 @@
 #include <pwd.h>
 #include <grp.h>
 #include <time.h>
+#include <limits.h>
 #include "aureport-options.h"
 #include "ausearch-time.h"
 #include "libaudit.h"
+#include "auparse-defs.h"
 
 
 /* Global vars that will be accessed by the main program */
@@ -44,6 +46,7 @@ int no_config = 0;
 /* These are for compatibility with parser */
 unsigned int event_id = -1;
 uid_t event_uid = -1, event_loginuid = -2, event_euid = -1;
+const char *event_tuid = NULL, *event_teuid = NULL, *event_tauid = NULL;
 gid_t event_gid = -1, event_egid = -1;
 slist *event_node_list = NULL;
 const char *event_key = NULL;
@@ -70,6 +73,7 @@ failed_t event_failed = F_BOTH;
 conf_act_t event_conf_act = C_NEITHER;
 success_t event_success = S_SUCCESS;
 int event_pid = 0;
+auparse_esc_t escape_mode = AUPARSE_ESC_TTY;
 
 struct nv_pair {
     int        value;
@@ -81,7 +85,7 @@ enum {  R_INFILE, R_TIME_END, R_TIME_START, R_VERSION, R_SUMMARY, R_LOG_TIMES,
 	R_AVCS, R_SYSCALLS, R_PIDS, R_EVENTS, R_ACCT_MODS,  
 	R_INTERPRET, R_HELP, R_ANOMALY, R_RESPONSE, R_SUMMARY_DET, R_CRYPTO,
 	R_MAC, R_FAILED, R_SUCCESS, R_ADD, R_DEL, R_AUTH, R_NODE, R_IN_LOGS,
-	R_KEYS, R_TTY, R_NO_CONFIG, R_COMM, R_VIRT, R_INTEG };
+	R_KEYS, R_TTY, R_NO_CONFIG, R_COMM, R_VIRT, R_INTEG, R_ESCAPE };
 
 static struct nv_pair optiontab[] = {
 	{ R_AUTH, "-au" },
@@ -97,6 +101,7 @@ static struct nv_pair optiontab[] = {
 	{ R_DEL, "--delete" },
 	{ R_EVENTS, "-e" },
 	{ R_EVENTS, "--event" },
+	{ R_ESCAPE, "--escape" },
 	{ R_FILES, "-f" },
 	{ R_FILES, "--file" },
 	{ R_FAILED, "--failed" },
@@ -254,6 +259,13 @@ int check_params(int count, char *vars[])
 					vars[c]);
 				retval = -1;
 			} else {
+				if (strlen(optarg) >= PATH_MAX) {
+					fprintf(stderr, 
+						"File name is too long %s\n",
+						optarg);
+					retval = -1;
+					break;
+				}
 				user_file = strdup(optarg);
 				if (user_file == NULL)
 					retval = -1;
@@ -293,6 +305,7 @@ int check_params(int count, char *vars[])
 			else { 
 				set_detail(D_DETAILED);
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_INTEG:
@@ -301,6 +314,7 @@ int check_params(int count, char *vars[])
 			else { 
 				set_detail(D_DETAILED);
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_VIRT:
@@ -316,6 +330,7 @@ int check_params(int count, char *vars[])
 			else { 
 				set_detail(D_DETAILED);
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_CRYPTO:
@@ -324,6 +339,7 @@ int check_params(int count, char *vars[])
 			else { 
 				set_detail(D_DETAILED);
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_LOGINS:
@@ -335,6 +351,7 @@ int check_params(int count, char *vars[])
 				event_hostname = dummy;
 				event_terminal = dummy;
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_ACCT_MODS:
@@ -346,6 +363,7 @@ int check_params(int count, char *vars[])
 				event_hostname = dummy;
 				event_terminal = dummy;
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 			break;
 		case R_EVENTS:
@@ -355,6 +373,7 @@ int check_params(int count, char *vars[])
 //				if (!optarg) {
 					set_detail(D_DETAILED);
 					event_loginuid = 1;
+					event_tauid = dummy;
 //				} else {
 //					UNIMPLEMENTED;
 //					set_detail(D_SPECIFIC);
@@ -386,6 +405,7 @@ int check_params(int count, char *vars[])
 					event_filename = dummy;
 					event_exe = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -399,6 +419,7 @@ int check_params(int count, char *vars[])
 					set_detail(D_DETAILED);
 					event_hostname = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -421,6 +442,7 @@ int check_params(int count, char *vars[])
 					set_detail(D_DETAILED);
 					event_exe = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -434,6 +456,7 @@ int check_params(int count, char *vars[])
 					set_detail(D_DETAILED);
 					event_comm = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -449,6 +472,7 @@ int check_params(int count, char *vars[])
 					event_hostname = dummy;
 					event_exe = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -465,6 +489,7 @@ int check_params(int count, char *vars[])
 					event_exe = dummy;
 					event_uid = 1;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -480,6 +505,7 @@ int check_params(int count, char *vars[])
 					event_hostname = dummy;
 					event_exe = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -495,6 +521,7 @@ int check_params(int count, char *vars[])
 					event_hostname = dummy;
 					event_comm = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -511,6 +538,7 @@ int check_params(int count, char *vars[])
 					event_exe = dummy;
 					event_comm = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -536,6 +564,7 @@ int check_params(int count, char *vars[])
 					event_exe = dummy;
 					event_key = dummy;
 					event_loginuid = 1;
+					event_tauid = dummy;
 				} else {
 					UNIMPLEMENTED;
 				}
@@ -548,6 +577,7 @@ int check_params(int count, char *vars[])
 				set_detail(D_DETAILED);
 				event_session_id = 1;
 				event_loginuid = 1;
+				event_tauid = dummy;
 				event_terminal = dummy;
 				event_comm = dummy;
 			}
@@ -661,6 +691,31 @@ int check_params(int count, char *vars[])
 				slist_append(event_node_list, &sn);
 			}
 			break;
+		case R_ESCAPE:
+			if (!optarg) {
+				fprintf(stderr, 
+					"Argument is required for %s\n",
+					vars[c]);
+				retval = -1;
+			} else {
+				if (strcmp(optarg, "raw") == 0)
+					escape_mode = AUPARSE_ESC_RAW;
+				else if (strcmp(optarg, "tty") == 0)
+					escape_mode = AUPARSE_ESC_TTY;
+				else if (strncmp(optarg, "shell", 6) == 0)
+					escape_mode = AUPARSE_ESC_SHELL;
+				else if (strcmp(optarg, "shell_quote") == 0)
+					escape_mode = AUPARSE_ESC_SHELL_QUOTE;
+				else {
+					fprintf(stderr, 
+						"Unknown option (%s)\n",
+						optarg);
+					retval = -1;
+					break;
+				}
+				c++;
+			}
+			break;
 		case R_SUMMARY_DET:
 			set_detail(D_SUM);
 			break;
@@ -712,6 +767,7 @@ int check_params(int count, char *vars[])
 				event_comm = dummy;
 				event_key = dummy;
 				event_loginuid = 1;
+				event_tauid = dummy;
 			}
 		}
 	} else
