@@ -38,7 +38,7 @@
 static void output_raw(llist *l);
 static void output_default(llist *l);
 static void output_interpreted(llist *l);
-static void output_interpreted_node(const lnode *n, const event *e);
+static void output_interpreted_record(const lnode *n, const event *e);
 static void feed_auparse(llist *l, auparse_callback_ptr callback);
 static void interpret(char *name, char *val, int comma, int rtype);
 static void csv_event(auparse_state_t *au,
@@ -159,10 +159,10 @@ static void output_interpreted(llist *l)
 		return;
 	}
 	if (n->type >= AUDIT_DAEMON_START && n->type < AUDIT_SYSCALL) 
-		output_interpreted_node(n, &(l->e));
+		output_interpreted_record(n, &(l->e));
 	else {
 		do {
-			output_interpreted_node(n, &(l->e));
+			output_interpreted_record(n, &(l->e));
 		} while ((n=list_prev(l)));
 	}
 }
@@ -171,7 +171,7 @@ static void output_interpreted(llist *l)
  * This function will cycle through a single record and lookup each field's
  * value that it finds. 
  */
-static void output_interpreted_node(const lnode *n, const event *e)
+static void output_interpreted_record(const lnode *n, const event *e)
 {
 	char *ptr, *str = n->message;
 	int found, comma = 0;
@@ -229,7 +229,7 @@ no_print:
 	else
 		strcpy(tmp, "?");
 	printf("%s", tmp);
-	printf(".%03d:%lu) ", e->milli, e->serial);
+	printf(".%03u:%lu) ", e->milli, e->serial);
 
 	if (n->type == AUDIT_SYSCALL) { 
 		a0 = n->a0;
@@ -372,6 +372,7 @@ static void interpret(char *name, char *val, int comma, int rtype)
 	id.a1 = a1;
 	id.name = name;
 	id.val = val;
+	id.cwd = NULL;
 
 	char *out = auparse_do_interpretation(type, &id, escape_mode);
 	if (type == AUPARSE_TYPE_UNCLASSIFIED)
@@ -530,7 +531,7 @@ static void csv_event(auparse_state_t *au,
 	// SUBJ_PRIME
 	rc = auparse_normalize_subject_primary(au);
 	if (rc == 1) {
-		char *subj = auparse_interpret_field(au);
+		const char *subj = auparse_interpret_field(au);
 		if (strcmp(subj, "unset") == 0)
 			subj = "system";
 		printf("%s", subj);
@@ -586,8 +587,15 @@ static void csv_event(auparse_state_t *au,
 
 	// OBJ_PRIME
 	rc = auparse_normalize_object_primary(au);
-	if (rc == 1)
-		printf("%s", auparse_interpret_field(au));
+	if (rc == 1) {
+		const char *val;
+
+		if (auparse_get_field_type(au) == AUPARSE_TYPE_ESCAPED_FILE)
+			val = auparse_interpret_realpath(au);
+		else
+			val = auparse_interpret_field(au);
+		printf("%s", val);
+	}
 	putchar(',');
 
 	// OBJ_SEC
@@ -699,10 +707,17 @@ static void text_event(auparse_state_t *au,
 
 	rc = auparse_normalize_object_primary(au);
 	if (rc == 1) {
+		const char *val;
+
 		// If we have an object and this is an AVC, add some words
 		if (action && strstr(action, "violated"))
 			printf("accessing ");
-		printf("%s ", auparse_interpret_field(au));
+
+		if (auparse_get_field_type(au) == AUPARSE_TYPE_ESCAPED_FILE)
+			val = auparse_interpret_realpath(au);
+		else
+			val = auparse_interpret_field(au);
+		printf("%s ", val);
 	}
 
 	if (    type == AUDIT_VIRT_RESOURCE ||
